/*----------------------------------------------------------------------------

   libtunepimp -- The MusicBrainz tagging library.  
                  Let a thousand taggers bloom!
   
   Copyright (C) Robert Kaye 2003
   
   This file is part of libtunepimp.

   libtunepimp is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   libtunepimp is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with libtunepimp; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   $Id: c_wrapper.cpp,v 1.12 2003/07/25 23:23:21 robert Exp $

----------------------------------------------------------------------------*/
#include "tunepimp.h"
#include "tp_c.h"

#define DB printf("%s:%d\n", __FILE__, __LINE__);

#define TP_OBJ_CHECK(o) TunePimp *obj = (TunePimp *)o; \
                        if (obj == NULL) return 0;
#define TP_OBJ_CHECKV(o) TunePimp *obj = (TunePimp *)o; \
                         if (obj == NULL) return;
#define TP_OBJ_CHECKN(o) TunePimp *obj = (TunePimp *)o; \
                         if (obj == NULL) return NULL;
#define TR_OBJ_CHECK(o) Track *obj = (Track *)t; \
                        if (obj == NULL) return 0;
#define TR_OBJ_CHECKV(o) Track *obj = (Track *)t; \
                         if (obj == NULL) return;

extern "C"
{

class Callback : public TPCallback
{
    public:

                 Callback(TunePimp *pimp) 
        { 
            tunePimp = pimp; 
            notifyCallback = NULL;
            statusCallback = NULL;
        };
        virtual ~Callback(void) {};

        void notify(TPCallbackEnum type, int fileId)
        {
            if (notifyCallback)
                (*notifyCallback)(type, fileId);
        };
        void status(const string &status) 
        {
            if (statusCallback)
                (*statusCallback)(status.c_str());
        };
       
        tp_notify_callback notifyCallback;
        tp_status_callback statusCallback;

    private:

        TunePimp           *tunePimp;
};

tunepimp_t tp_New(const char *appName, const char *appVersion)
{
    TunePimp *pimp = new TunePimp(appName, appVersion);
    Callback *cb = new Callback(pimp);

    pimp->setCallback(cb);

    return (tunepimp_t)pimp;
}

void tp_Delete(tunepimp_t o)
{
    Callback *old;

    TP_OBJ_CHECKV(o);

    old = (Callback *)obj->getCallback();
    delete (TunePimp *)obj; 
    delete old;
}

void tp_SetNotifyCallback(tunepimp_t o, tp_notify_callback callback)
{
    Callback *cb;

    TP_OBJ_CHECKV(o);

    cb = (Callback *)obj->getCallback();
    cb->notifyCallback = callback;
}

void tp_SetStatusCallback(tunepimp_t o, tp_status_callback callback)
{
    Callback *cb;

    TP_OBJ_CHECKV(o);

    cb = (Callback *)obj->getCallback();
    cb->statusCallback = callback;
}

void tp_GetVersion(tunepimp_t o, int *major, int *minor, int *rev)
{
    *major = *minor = *rev = 0;

    TP_OBJ_CHECKV(o);

    obj->getVersion(*major, *minor, *rev);
}

void tp_SetServer(tunepimp_t o, const char *serverAddr, short serverPort)
{
    TP_OBJ_CHECKV(o);

    obj->setServer(string(serverAddr), serverPort);
}

void tp_SetDebug(tunepimp_t o, int debug)
{
    TP_OBJ_CHECKV(o);

    obj->setDebug((bool)debug);
}

void tp_SetProxy(tunepimp_t o,char *proxyAddr, short proxyPort)
{
    TP_OBJ_CHECKV(o);

    string addr = "";
    if (proxyAddr) 
        addr = proxyAddr;
    obj->setProxy(addr, proxyPort);
}

void tp_UseUTF8(tunepimp_t o, int UTF8)
{
    TP_OBJ_CHECKV(o);

    obj->setUseUTF8((bool)UTF8);
}

void tp_SetUserInfo(tunepimp_t o, const char *userName, const char *password)
{
    TP_OBJ_CHECKV(o);

    obj->setUserInfo(userName, password);
}

int tp_GetNumSupportedExtensions(tunepimp_t o)
{
    vector<string> extList;

    TP_OBJ_CHECK(o);

    obj->getSupportedExtensions(extList);

    return extList.size();
}

void tp_GetSupportedExtensions(tunepimp_t o, char extensions[][32])
{
    vector<string>            extList;
    vector<string>::iterator  i;
    int                       count;

    TP_OBJ_CHECKV(o);

    obj->getSupportedExtensions(extList);
    for(i = extList.begin(), count = 0; i != extList.end(); i++, count++)
       strcpy(extensions[count], (*i).c_str());
}

void tp_SetAutoFileLookup(tunepimp_t o, int enable)
{
    TP_OBJ_CHECKV(o);

    obj->setAutoFileLookup((bool)enable);
}


void tp_GetError(tunepimp_t o, char *error, int maxLen)
{
    string err;

    TP_OBJ_CHECKV(o);

    obj->getError(err);
    strncpy(error, err.c_str(), maxLen - 1);
    error[maxLen - 1] = 0;
}

int tp_Add(tunepimp_t o, const char *fileName)
{
    TP_OBJ_CHECK(o);

    return (int)obj->add(fileName);
}

int tp_AddDir(tunepimp_t o, const char *dirPath)
{
    TP_OBJ_CHECK(o);
    return (int)obj->addDir(dirPath);
}

void tp_Remove(tunepimp_t o, int fileId)
{
    TP_OBJ_CHECKV(o);
    obj->remove(fileId);
}

void tp_Wake(tunepimp_t o, track_t track)
{
    TP_OBJ_CHECKV(o);

    obj->wake((Track *)track);
}

int tp_GetNumFiles(tunepimp_t o)
{
    TP_OBJ_CHECK(o);

    return obj->getNumFiles();
}

int tp_GetNumUnsubmitted(tunepimp_t o)
{
    TP_OBJ_CHECK(o);

    return obj->getNumUnsubmitted();
}

int tp_GetNumFileIds(tunepimp_t o)
{
    vector<int> ids;

    TP_OBJ_CHECK(o);

    obj->getFileIds(ids);
    return ids.size();
}

void tp_GetFileIds(tunepimp_t o, int *ids)
{
    vector<int>           vec;
    vector<int>::iterator i;

    TP_OBJ_CHECKV(o);

    obj->getFileIds(vec);
    for(i = vec.begin(); i != vec.end(); i++, ids++)
        *ids = *i;
}

track_t tp_GetTrack(tunepimp_t o, int fileId)
{
    TP_OBJ_CHECKN(o);

    return (track_t)obj->getTrack(fileId);
}

void tp_ReleaseTrack(tunepimp_t o, track_t track)
{
    TP_OBJ_CHECKV(o);

    if (track == NULL)
        return;

    obj->releaseTrack((Track *)track);
}

TPError tp_SelectResult(tunepimp_t o, track_t track, int resultIndex)
{
    TunePimp *obj = (TunePimp *)o; 
    if (o == NULL) 
        return tpInvalidObject;

    if (track == NULL)
        return tpInvalidObject;

    obj->selectResult((Track *)track, resultIndex);

    return tpOk;
}

void tp_Misidentified(tunepimp_t o, int fileId)
{
    TP_OBJ_CHECKV(o);

    obj->misidentified(fileId);
}

void tp_IdentifyAgain(tunepimp_t o, int fileId)
{
    TP_OBJ_CHECKV(o);

    obj->identifyAgain(fileId);
}

int tp_WriteTags(tunepimp_t o, int *fileIds, int numFileIds)
{
    vector<int> ids;

    TP_OBJ_CHECK(o);

    if (!fileIds)
        return obj->writeTags(NULL);

    for(; numFileIds >= 0; numFileIds--, fileIds++)
        ids.push_back(*fileIds);

    return obj->writeTags(&ids);
}

TPError tp_SubmitTRMs(tunepimp_t o)
{
    TunePimp *obj = (TunePimp *)o; 
    if (o == NULL) 
        return tpInvalidObject;

    return obj->submitTRMs();
}

void tp_SetRenameFiles(tunepimp_t o, int rename)
{
    TP_OBJ_CHECKV(o);

    obj->setRenameFiles((bool)rename);
}

void tp_SetMoveFiles(tunepimp_t o, int move)
{
    TP_OBJ_CHECKV(o);

    obj->setMoveFiles((bool)move);
}

void tp_SetWriteID3v1(tunepimp_t o, int writeID3v1)
{
    TP_OBJ_CHECKV(o);

    obj->setWriteID3v1((bool)writeID3v1);
}

void tp_SetClearTags(tunepimp_t o, int clearTags)
{
    TP_OBJ_CHECKV(o);

    obj->setClearTags((bool)clearTags);
}

void tp_SetFileMask(tunepimp_t o, const char *fileMask)
{
    TP_OBJ_CHECKV(o);

    obj->setFileMask(fileMask);
}

void tp_SetVariousFileMask(tunepimp_t o, const char *variousFileMask)
{
    TP_OBJ_CHECKV(o);

    obj->setVariousFileMask(variousFileMask);
}

void tp_SetAllowedFileCharacters(tunepimp_t o, const char *allowedCharacters)
{
    TP_OBJ_CHECKV(o);

    obj->setAllowedFileCharacters(allowedCharacters);
}

void tp_SetDestDir(tunepimp_t o, const char *destDir)
{
    TP_OBJ_CHECKV(o);

    obj->setDestDir(destDir);
}

void tp_SetTopSrcDir(tunepimp_t o, const char *topSrcDir)
{
    TP_OBJ_CHECKV(o);

    obj->setTopSrcDir(topSrcDir);
}

void tp_SetTRMCollisionThreshold(tunepimp_t o, int trmThreshold)
{
    TP_OBJ_CHECKV(o);

    obj->setTRMCollisionThreshold(trmThreshold);
}

void tp_SetAutoSaveThreshold(tunepimp_t o, int autoSaveThreshold)
{
    TP_OBJ_CHECKV(o);

    obj->setAutoSaveThreshold(autoSaveThreshold);
}

#ifdef WIN32
void tp_WSAInit(tunepimp_t o)
{
    TP_OBJ_CHECKV(o);

    obj->WSAInit();
}
    
void tp_WSAStop(tunepimp_t o)
{
    TP_OBJ_CHECKV(o);

    obj->WSAStop();
}
#endif

/* --------------------------------------------------------------------------
 * Track Interface
 * --------------------------------------------------------------------------*/

TPFileStatus tr_GetStatus(track_t t)
{
    Track *obj = (Track *)t; 
    if (t == NULL) 
        return eError;

    return obj->getStatus();
}

void tr_GetFileName(track_t t, char *fileName, int maxLen)
{
    string file;

    TR_OBJ_CHECKV(t);

    obj->getFileName(file);
    strncpy(fileName, file.c_str(), maxLen - 1);
    fileName[maxLen - 1] = 0;
}

void tr_GetTRM(track_t t, char *trmArg, int maxLen)
{
    string trm;

    TR_OBJ_CHECKV(t);

    obj->getTRM(trm);
    strncpy(trmArg, trm.c_str(), maxLen - 1);
    trmArg[maxLen - 1] = 0;
}

void tr_GetLocalMetadata(track_t t, metadata_t *mdata)
{
    Metadata data;

    TR_OBJ_CHECKV(t);

    obj->getLocalMetadata(data);
    md_Clear(mdata);

    mdata->artist = strdup(data.artist.c_str());
    mdata->sortName = strdup(data.sortName.c_str());
    mdata->album = strdup(data.album.c_str());
    mdata->track = strdup(data.track.c_str());
    mdata->trackNum = data.trackNum;
    mdata->variousArtist = (bool)data.variousArtist;
    mdata->artistId = strdup(data.artistId.c_str());
    mdata->albumId = strdup(data.albumId.c_str());
    mdata->trackId = strdup(data.trackId.c_str());
    mdata->fileTrm = strdup(data.fileTrm.c_str());
    mdata->albumArtistId = strdup(data.albumArtistId.c_str());
    mdata->duration = data.duration;
    mdata->albumType = data.albumType;
    mdata->numTRMIds = data.numTRMIds;
}

void tr_SetLocalMetadata(track_t t, const metadata_t *mdata)
{
    Metadata data;

    TR_OBJ_CHECKV(t);

    data.artist = mdata->artist;
    data.sortName = mdata->sortName;
    data.album = mdata->album;
    data.track = mdata->track;
    data.trackNum = mdata->trackNum;
    data.variousArtist = (bool)mdata->variousArtist;
    data.artistId = mdata->artistId;
    data.albumId = mdata->albumId;
    data.trackId = mdata->trackId;
    data.fileTrm = mdata->fileTrm;
    data.albumArtistId = mdata->albumArtistId;
    data.duration = mdata->duration;
    data.albumType = mdata->albumType;
    data.numTRMIds = mdata->numTRMIds;

    obj->setLocalMetadata(data);
}

void tr_GetServerMetadata(track_t t, metadata_t *mdata)
{
    Metadata data;

    TR_OBJ_CHECKV(t);

    obj->getServerMetadata(data);
    md_Clear(mdata);

    mdata->artist = strdup(data.artist.c_str());
    mdata->sortName = strdup(data.sortName.c_str());
    mdata->album = strdup(data.album.c_str());
    mdata->track = strdup(data.track.c_str());
    mdata->trackNum = data.trackNum;
    mdata->variousArtist = (bool)data.variousArtist;
    mdata->artistId = strdup(data.artistId.c_str());
    mdata->albumId = strdup(data.albumId.c_str());
    mdata->trackId = strdup(data.trackId.c_str());
    mdata->fileTrm = strdup(data.fileTrm.c_str());
    mdata->albumArtistId = strdup(data.albumArtistId.c_str());
    mdata->duration = data.duration;
    mdata->albumType = data.albumType;
    mdata->numTRMIds = data.numTRMIds;

}

void tr_SetServerMetadata(track_t t, const metadata_t *mdata)
{
    Metadata data;

    TR_OBJ_CHECKV(t);

    data.artist = mdata->artist;
    data.sortName = mdata->sortName;
    data.album = mdata->album;
    data.track = mdata->track;
    data.trackNum = mdata->trackNum;
    data.variousArtist = (bool)mdata->variousArtist;
    data.artistId = mdata->artistId;
    data.albumId = mdata->albumId;
    data.trackId = mdata->trackId;
    data.fileTrm = mdata->fileTrm;
    data.albumArtistId = mdata->albumArtistId;
    data.duration = mdata->duration;
    data.albumType = mdata->albumType;
    data.numTRMIds = mdata->numTRMIds;

    obj->setServerMetadata(data);
}

void tr_GetError(track_t t, char *error, int maxLen)
{
    string err;

    TR_OBJ_CHECKV(t);

    obj->getError(err);
    strncpy(error, err.c_str(), maxLen - 1);
    error[maxLen - 1] = 0;
}

int tr_GetSimilarity(track_t t)
{
    TR_OBJ_CHECK(t);
    return obj->getSimilarity();
}

int tr_HasChanged(track_t t)
{
    TR_OBJ_CHECK(t);
    return (int)obj->hasChanged();
}

void tr_SetStatus(track_t t, const TPFileStatus status)
{
    TR_OBJ_CHECKV(t);

    obj->setStatus(status);
}

int tr_GetNumResults(track_t t)
{
    vector<TPResult *> res;
    TPResultType       type;

    TR_OBJ_CHECK(t);

    obj->getResults(type, res);
    return res.size();
}

static artistresult_t *convertArtistResult(TPArtistResult *res)
{
    artistresult_t *ar;

    ar = (artistresult_t *)calloc(sizeof(artistresult_t), 1);
    ar->relevance = res->getRelevance();
    ar->name = strdup(res->getName().c_str());
    ar->sortName = strdup(res->getSortName().c_str());
    ar->id = strdup(res->getId().c_str());

    //printf("Artist res: %d %s %s %s\n", ar->relevance, ar->name, ar->sortName, ar->id);

    return ar;
}

static void deleteArtistResult(artistresult_t *res)
{
    if (!res)
        return;

    if (res->name)
        free(res->name);

    if (res->sortName)
        free(res->sortName);

    if (res->id)
        free(res->id);

    free(res);
}

static albumresult_t *convertAlbumResult(TPAlbumResult *res)
{
    albumresult_t *al;
    TPArtistResult artist;

    al = (albumresult_t *)calloc(sizeof(albumresult_t), 1);
    al->relevance = res->getRelevance();
    al->name = strdup(res->getName().c_str());
    al->id = strdup(res->getId().c_str());
    al->numTracks = res->getNumTracks();
    al->numCDIndexIds = res->getNumCDIndexIds();
    al->isVA = (int)res->getVariousArtists();
    al->status = res->getStatus();
    al->type = res->getType();

    artist = res->getArtist();
    al->artist = convertArtistResult(&artist);

    //printf("Album res: %d %s %s\n", al->relevance, al->name, al->id);

    return al;
}

static void deleteAlbumResult(albumresult_t *res)
{
    if (!res)
        return;

    if (res->name)
        free(res->name);

    if (res->id)
        free(res->id);

    free(res);
}

static albumtrackresult_t *convertAlbumTrackResult(TPAlbumTrackResult *res)
{
    albumtrackresult_t *tr;
    TPArtistResult     artist;
    TPAlbumResult      album;

    tr = (albumtrackresult_t *)calloc(sizeof(albumtrackresult_t), 1);
    tr->relevance = res->getRelevance();
    tr->name = strdup(res->getName().c_str());
    tr->id = strdup(res->getId().c_str());
    tr->numTRMIds = res->getNumTRMIds();
    tr->trackNum = res->getTrackNum();
    tr->duration = res->getDuration();

    artist = res->getArtist();
    tr->artist = convertArtistResult(&artist);

    album = res->getAlbum();
    tr->album = convertAlbumResult(&album);

    return tr;
}

static void deleteAlbumTrackResult(albumtrackresult_t *res)
{
    if (!res)
        return;

    if (res->name)
        free(res->name);

    if (res->id)
        free(res->id);

    free(res);
}

void tr_GetResults(track_t t, TPResultType *type, result_t *results, int numResults)
{
    vector<TPResult *> res;
    vector<TPResult *>::iterator i;

    TR_OBJ_CHECKV(t);

    obj->getResults(*type, res);
    for(i = res.begin(); i != res.end() && numResults >= 0; i++, numResults--, results++)
    {
        switch(*type)
        {
            case eArtistList:
                *results = convertArtistResult((TPArtistResult *)(*i));
                break;
            case eAlbumList:
                *results = convertAlbumResult((TPAlbumResult *)(*i));
                break;
            case eTrackList:
                *results = convertAlbumTrackResult((TPAlbumTrackResult *)(*i));
                break;
            default:
                *results = NULL;
        }
    }
}

void tr_Lock(track_t t)
{
    TR_OBJ_CHECKV(t);
    obj->lock();
}

void tr_Unlock(track_t t)
{
    TR_OBJ_CHECKV(t);
    obj->unlock();
}

/* --------------------------------------------------------------------------
 * Result Interface
 * --------------------------------------------------------------------------*/

void rs_Delete(TPResultType *type, result_t *result)
{
    switch(*type)
    {
        case eArtistList:
            deleteArtistResult((artistresult_t *)result);
            break;
        case eAlbumList:
            deleteAlbumResult((albumresult_t *)result);
            break;
        case eTrackList:
            deleteAlbumTrackResult((albumtrackresult_t *)result);
            break;
        default:
            break;
    }
}

/* --------------------------------------------------------------------------
 * Metadata Interface
 * --------------------------------------------------------------------------*/

metadata_t *md_New(void)
{
    return (metadata_t *)calloc(sizeof(metadata_t), 1);
}

void md_Delete(metadata_t *data)
{
    if (!data)
        return;

    md_Clear(data);

    free(data);
}

void md_Clear(metadata_t *data)
{
    if (!data)
        return;

    if (data->artist)
        free(data->artist);
    if (data->sortName)
        free(data->sortName);
    if (data->album)
        free(data->album);
    if (data->track)
        free(data->track);
    if (data->artistId)
        free(data->artistId);
    if (data->albumId)
        free(data->albumId);
    if (data->trackId)
        free(data->trackId);
    if (data->fileTrm)
        free(data->fileTrm);
    if (data->albumArtistId)
        free(data->albumArtistId);

    memset(data, 0, sizeof(metadata_t));
}

TPAlbumStatus md_ConvertToAlbumStatus(const char *albumStatus)
{
    return convertToAlbumStatus(albumStatus);
}

TPAlbumType md_ConvertToAlbumType(const char *albumType)
{
    return convertToAlbumType(albumType);
}

void md_ConvertFromAlbumStatus(TPAlbumStatus status, char *albumStatus, int maxLen)
{
    string statusStr;

    convertFromAlbumStatus(status, statusStr);
    strncpy(albumStatus, statusStr.c_str(), maxLen - 1);
    albumStatus[maxLen - 1] = 0;
}

void md_ConvertFromAlbumType(TPAlbumType type, char *albumType, int maxLen)
{
    string typeStr;

    convertFromAlbumType(type, typeStr);
    strncpy(albumType, typeStr.c_str(), maxLen - 1);
    albumType[maxLen - 1] = 0;
}

}
